@@ -41,8 +41,8 @@
 
 #include <string>
 #include <vector>
+#include <memory>
 
-#include "SharedHandle.h"
 #include "error_code.h"
 #include "RequestGroup.h"
 
@@ -52,48 +52,48 @@ class Option;
 class FileEntry;
 class MetadataInfo;
 
-struct DownloadResult
-{
-  a2_gid_t gid;
- 
-  std::vector<SharedHandle<FileEntry> > fileEntries;
-
-  bool inMemoryDownload;
+struct DownloadResult {
+  // This field contains GID. See comment in
+  // RequestGroup.cc::belongsToGID_.
+  a2_gid_t belongsTo;
 
   uint64_t sessionDownloadLength;
 
-  // milliseconds
-  int64_t sessionTime;
+  std::chrono::milliseconds sessionTime;
 
-  error_code::Value result;
+  int64_t totalLength;
+
+  int64_t completedLength;
+
+  int64_t uploadLength;
+
+  std::shared_ptr<GroupId> gid;
+
+  std::shared_ptr<Option> option;
+
+  std::shared_ptr<MetadataInfo> metadataInfo;
+
+  std::vector<std::shared_ptr<FileEntry>> fileEntries;
 
   // This field contains GIDs. See comment in
   // RequestGroup.cc::followedByGIDs_.
   std::vector<a2_gid_t> followedBy;
 
-  // This field contains GID. See comment in
-  // RequestGroup.cc::belongsToGID_.
-  a2_gid_t belongsTo;
+  std::string bitfield;
 
-  SharedHandle<Option> option;
+  std::string infoHash;
 
-  SharedHandle<MetadataInfo> metadataInfo;
-
-  uint64_t totalLength;
-
-  uint64_t completedLength;
-
-  uint64_t uploadLength;
-
-  std::string bitfieldStr;
-
-  std::string infoHashStr;
-
-  size_t pieceLength;
+  std::string dir;
 
   size_t numPieces;
 
-  std::string dir;
+  int32_t pieceLength;
+
+  error_code::Value result;
+
+  std::string resultMessage;
+
+  bool inMemoryDownload;
 
   DownloadResult();
   ~DownloadResult();
@@ -103,8 +103,6 @@ struct DownloadResult
   DownloadResult& operator=(const DownloadResult& c);
 };
 
-typedef SharedHandle<DownloadResult> DownloadResultHandle;
-
 } // namespace aria2
 
 #endif // D_DOWNLOAD_RESULT_H
